@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#    https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -133,6 +133,42 @@ def display_bloch_spheres_and_correlations(
         C(| \psi \rangle) = 2 \left| \langle 00 | \psi \rangle
         \langle 11 | \psi \rangle - \langle 01 | \psi \rangle
         \langle 10 | \psi \rangle \right|.
+
+    .. rubric:: Styling
+
+    This function displays HTML and JavaScript in your Jupyter notebook. It
+    creates four HTML elements that you can style by defining the following
+    CSS classes:
+
+    qctrlvisualizer
+        CSS class for the outer ``<div>`` that contains the Bloch spheres
+        visualization, progress bar, and button. The div has inline styles
+        ``display:flex``, ``flex-direction:column``, and
+        ``align-items:center``, so if you want to use different values for
+        these properties you must either strip them out or use the
+        ``!important`` rule when defining your ``qctrlvisualizer`` class.
+
+    qctrlvisualizer-wrapper
+        CSS class for the ``<div>`` that contains the Bloch spheres
+        visualization. The div has inline styles ``margin:0.5rem 0``,
+        ``width``, and ``height`` (based on the `width` you pass).
+
+    qctrlvisualizer-progress-bar
+        CSS class for the range ``<input>`` representing the progress bar.
+        The input has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button
+        CSS class for the ``<button>`` representing the play/pause/replay
+        button. The button has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button-play
+        CSS class for the ``<button>`` when the button text is "Play".
+
+    qctrlvisualizer-button-pause
+        CSS class for the ``<button>`` when the button text is "Pause".
+
+    qctrlvisualizer-button-replay
+        CSS class for the ``<button>`` when the button text is "Replay".
     """
 
     identifier = uuid.uuid4().hex
@@ -293,6 +329,42 @@ def display_bloch_sphere(
     The Bloch sphere has unit radius. If the states that you provide are
     not normalized, then :math:`r \ne 1` and you can draw trajectories
     inside the Bloch sphere.
+
+    .. rubric:: Styling
+
+    This function displays HTML and JavaScript in your Jupyter notebook. It
+    creates four HTML elements that you can style by defining the following
+    CSS classes:
+
+    qctrlvisualizer
+        CSS class for the outer ``<div>`` that contains the Bloch sphere
+        visualization, progress bar, and button. The div has inline styles
+        ``display:flex``, ``flex-direction:column``, and
+        ``align-items:center``, so if you want to use different values for
+        these properties you must either strip them out or use the
+        ``!important`` rule when defining your ``qctrlvisualizer`` class.
+
+    qctrlvisualizer-wrapper
+        CSS class for the ``<div>`` that contains the Bloch sphere
+        visualization. The div has inline styles ``margin:0.5rem 0``,
+        ``width``, and ``height`` (based on the `width` you pass).
+
+    qctrlvisualizer-progress-bar
+        CSS class for the range ``<input>`` representing the progress bar.
+        The input has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button
+        CSS class for the ``<button>`` representing the play/pause/replay
+        button. The button has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button-play
+        CSS class for the ``<button>`` when the button text is "Play".
+
+    qctrlvisualizer-button-pause
+        CSS class for the ``<button>`` when the button text is "Pause".
+
+    qctrlvisualizer-button-replay
+        CSS class for the ``<button>`` when the button text is "Replay".
     """
     density_matrices = (
         np.asarray(states)[..., None] @ np.asarray(states)[..., None, :].conj()
@@ -372,6 +444,42 @@ def display_bloch_sphere_from_density_matrices(
         b_k = \mathrm{Tr} \left\{ \sigma_k \rho \right\},
 
     which is just the expectation value of :math:`\sigma_k`.
+
+    .. rubric:: Styling
+
+    This function displays HTML and JavaScript in your Jupyter notebook. It
+    creates four HTML elements that you can style by defining the following
+    CSS classes:
+
+    qctrlvisualizer
+        CSS class for the outer ``<div>`` that contains the Bloch sphere
+        visualization, progress bar, and button. The div has inline styles
+        ``display:flex``, ``flex-direction:column``, and
+        ``align-items:center``, so if you want to use different values for
+        these properties you must either strip them out or use the
+        ``!important`` rule when defining your ``qctrlvisualizer`` class.
+
+    qctrlvisualizer-wrapper
+        CSS class for the ``<div>`` that contains the Bloch sphere
+        visualization. The div has inline styles ``margin:0.5rem 0``,
+        ``width``, and ``height`` (based on the `width` you pass).
+
+    qctrlvisualizer-progress-bar
+        CSS class for the range ``<input>`` representing the progress bar.
+        The input has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button
+        CSS class for the ``<button>`` representing the play/pause/replay
+        button. The button has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button-play
+        CSS class for the ``<button>`` when the button text is "Play".
+
+    qctrlvisualizer-button-pause
+        CSS class for the ``<button>`` when the button text is "Pause".
+
+    qctrlvisualizer-button-replay
+        CSS class for the ``<button>`` when the button text is "Replay".
     """
     bloch_vectors = _expectation_values(
         np.asarray(density_matrices), np.asarray(PAULIS)
@@ -489,6 +597,42 @@ def display_bloch_sphere_from_bloch_vectors(
     .. math::
         \left| \psi \right\rangle = \cos(\theta/2) \left| 0 \right\rangle
         + e^{i \phi} \sin(\theta/2) \left| 1 \right\rangle.
+
+    .. rubric:: Styling
+
+    This function displays HTML and JavaScript in your Jupyter notebook. It
+    creates four HTML elements that you can style by defining the following
+    CSS classes:
+
+    qctrlvisualizer
+        CSS class for the outer ``<div>`` that contains the Bloch sphere
+        visualization, progress bar, and button. The div has inline styles
+        ``display:flex``, ``flex-direction:column``, and
+        ``align-items:center``, so if you want to use different values for
+        these properties you must either strip them out or use the
+        ``!important`` rule when defining your ``qctrlvisualizer`` class.
+
+    qctrlvisualizer-wrapper
+        CSS class for the ``<div>`` that contains the Bloch sphere
+        visualization. The div has inline styles ``margin:0.5rem 0``,
+        ``width``, and ``height`` (based on the `width` you pass).
+
+    qctrlvisualizer-progress-bar
+        CSS class for the range ``<input>`` representing the progress bar.
+        The input has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button
+        CSS class for the ``<button>`` representing the play/pause/replay
+        button. The button has inline style ``margin:0.5rem 0``.
+
+    qctrlvisualizer-button-play
+        CSS class for the ``<button>`` when the button text is "Play".
+
+    qctrlvisualizer-button-pause
+        CSS class for the ``<button>`` when the button text is "Pause".
+
+    qctrlvisualizer-button-replay
+        CSS class for the ``<button>`` when the button text is "Replay".
     """
     if np.iscomplexobj(bloch_vectors):
         raise ValueError("The Bloch vectors must not have complex type.")
@@ -566,6 +710,18 @@ def _display_visualization(
 
     theme_settings = {"highlightColor": color, "pathColor": color}
 
+    labels = {
+        "xAxis": True,
+        "yAxis": True,
+        "zAxis": True,
+        "theta": True,
+        "phi": True,
+        "northPole": True,
+        "southPole": True,
+        # Prevent "Non Error Trajectory" dot from appearing.
+        "nonErrorState": False,
+    }
+
     default_url = (
         "https://cdn.jsdelivr.net/npm/@qctrl/visualizer@3.1.11/umd/visualizer.min.js"
     )
@@ -574,12 +730,30 @@ def _display_visualization(
     visualizer_js = visualizer_js or default_url
 
     html_code = f"""
-    <div style="display: flex; flex-direction: column; align-items: center;">
-        <div id="wrapper{identifier}"
-             style="width: {width}px; height: {height}px;"></div>
-        <input id="scrub{identifier}"
-               type="range" min="0" max="1" step="0.01" value="0"/>
-        <button id="play{identifier}">Play</button>
+    <div
+        class="qctrlvisualizer"
+        style="display:flex;flex-direction:column;align-items:center;"
+    >
+        <div
+            class="qctrlvisualizer-wrapper"
+            style="width:{width}px;height:{height}px;margin:0.5rem 0"
+            id="qctrlvisualizer-wrapper-{identifier}"
+        ></div>
+        <input
+            class="qctrlvisualizer-progress-bar"
+            style="margin:0.5rem 0"
+            id="qctrlvisualizer-progress-bar-{identifier}"
+            type="range"
+            min="0"
+            max="1"
+            step="0.01"
+            value="0"
+        >
+        <button
+            class="qctrlvisualizer-button qctrlvisualizer-button-play"
+            style="margin:0.5rem 0"
+            id="qctrlvisualizer-button-{identifier}"
+        >Play</button>
     </div>
     """
 
@@ -589,33 +763,52 @@ def _display_visualization(
 
     const visualizationData = {json.dumps(visualization_data)};
     const themeSettings = {json.dumps(theme_settings)};
+    const labels = {json.dumps(labels)};
 
-    const wrapper = document.getElementById("wrapper{identifier}");
-    const playButton = document.getElementById("play{identifier}");
-    const scrub = document.getElementById("scrub{identifier}");
+    const wrapper = document.getElementById("qctrlvisualizer-wrapper-{identifier}");
+    const progressBar = document.getElementById("qctrlvisualizer-progress-bar-{identifier}");
+    const button = document.getElementById("qctrlvisualizer-button-{identifier}");
 
-    const originalButtonText = playButton.innerHTML;
+    function updateButton () {{
+      button.classList.remove(...[
+        "qctrlvisualizer-button-play",
+        "qctrlvisualizer-button-pause",
+        "qctrlvisualizer-button-replay"
+      ]);
+      if (isPlaying) {{
+        button.classList.add("qctrlvisualizer-button-pause");
+        button.innerHTML = "Pause";
+        return;
+      }}
+      if (progress>=1) {{
+        button.classList.add("qctrlvisualizer-button-replay");
+        button.innerHTML = "Replay";
+        return;
+      }}
+      button.classList.add("qctrlvisualizer-button-play");
+      button.innerHTML = "Play";
+    }}
 
-    playButton.onclick = () => {{
+    button.onclick = () => {{
       isPlaying = !isPlaying;
-      if (isPlaying) playButton.innerHTML = "Pause";
-      if (!isPlaying) playButton.innerHTML = originalButtonText;
       if (progress >= 1) progress = 0;
+      updateButton();
       visualizer.update({{ isPlaying, progress }});
     }};
 
-    scrub.oninput = ({{ target }}) => {{
+    progressBar.oninput = ({{ target }}) => {{
       progress = +target.value;
       visualizer.update({{ progress }});
+      updateButton();
     }};
 
     const onUpdate = ({{ target, data }}) => {{
       progress = data.progress;
-      scrub.value = progress;
+      progressBar.value = progress;
       if (progress >= 1) {{
         isPlaying = false;
         target.update({{ isPlaying }});
-        playButton.innerHTML = "Replay";
+        updateButton();
       }}
     }};
 
@@ -623,10 +816,56 @@ def _display_visualization(
       visualizationData,
       wrapper,
       onUpdate,
+      labels,
     }}).init();
 
     visualizer.update({{ themeSettings }});
     """
 
+    custom_js_loader = f"""
+    function visualizerLoader() {{
+      if ((typeof Visualizer) == "function") {{
+        console.log("Using preloaded Q-CTRL Visualizer JavaScript package.");
+        displayBlochSphere();
+        return;
+      }}
+
+      try {{
+        console.log("Attempting to load {visualizer_js} with require.js.");
+        requirejs(["{visualizer_js}"], displayBlochSphere, displayErrorMessage);
+      }} catch(error) {{
+        var existing_script = document.getElementById("qctrlvisualizer-script");
+
+        if (existing_script !== null) {{
+          console.log(
+            "Script tag for the Q-CTRL Visualizer JavaScript package already exists."
+            + " Delaying execution of function until script is loaded."
+          );
+          existing_script.addEventListener("load", displayBlochSphere);
+          existing_script.addEventListener("error", displayErrorMessage);
+          return;
+        }}
+
+        console.log("Attempting to load {visualizer_js} with script tag.");
+        script = document.createElement("script");
+        script.onload = displayBlochSphere;
+        script.onerror = displayErrorMessage;
+        script.id = "qctrlvisualizer-script";
+        script.src = "{visualizer_js}";
+        document.head.appendChild(script);
+      }}
+    }}
+
+    function displayBlochSphere() {{ {js_code} }}
+
+    function displayErrorMessage() {{
+      console.log("Failed to load {visualizer_js}.");
+      const wrapper = document.getElementById("qctrlvisualizer-wrapper-{identifier}");
+      wrapper.innerHTML = "Could not load JavaScript at {visualizer_js}.";
+    }}
+
+    visualizerLoader();
+    """
+
     display(HTML(html_code))
-    display(Javascript(js_code, lib=visualizer_js))
+    display(Javascript(custom_js_loader))
