@@ -81,18 +81,51 @@ def plot_controls(
     ------
     ValueError
         If any of the input parameters are invalid.
+    TypeError
+        If the controls have invalid types.
     """
+    boulder_opal_advice = (
+        "If you are attempting to plot controls from Boulder Opal, you can obtain"
+        " controls in the correct format by using Pwcs as the output nodes."
+    )
+
     plots_data: List[Any] = []
+
+    if not isinstance(controls, dict):
+        raise TypeError(
+            f"The `controls` parameter must be a dictionary. {boulder_opal_advice}"
+        )
+
     for name, control in controls.items():
         # Process each control depending whether it's passed as a list of segments...
         if isinstance(control, list):
             durations = []
             values = []
             for segment in control:
+                if not isinstance(segment, dict):
+                    raise TypeError(
+                        "Each control segment must be a dictionary."
+                        f" {boulder_opal_advice}"
+                    )
+                if ("duration" not in segment) or ("value" not in segment):
+                    raise ValueError(
+                        "Each control segment dictionary must contain `duration` and"
+                        f" `value` keys. {boulder_opal_advice}"
+                    )
                 durations.append(segment["duration"])
                 values.append(segment["value"])
         # ...or a durations/values dictionary.
         else:
+            if not isinstance(control, dict):
+                raise TypeError(
+                    "Each control must either be a list or a dictionary."
+                    f" {boulder_opal_advice}"
+                )
+            if ("durations" not in control) or ("values" not in control):
+                raise ValueError(
+                    "Each control dictionary must contain `durations` and `values`"
+                    f" keys. {boulder_opal_advice}"
+                )
             durations = control["durations"]
             values = control["values"]
         # Create plot data for the control.
