@@ -61,9 +61,12 @@ def get_qctrl_style() -> Dict[str, Any]:
     style["ytick.color"] = TEXT_COLOR
     style["axes.labelcolor"] = TEXT_COLOR
 
-    # Set default font to Roboto.
+    # Set font to Roboto.
     style["font.sans-serif"] = ["Roboto"] + rcParams["font.sans-serif"]
 
+    # Set figure size.
+    style["figure.figsize"] = [10.0, 5.0]
+
     # Set borders and their color.
     style["axes.spines.left"] = True
     style["axes.spines.right"] = True
